<?php

    /**
    * Project : Technical Project Manager (IEEE like)
    *
    * Task operations
    *
    * @package mod-techproject
    * @category mod
    * @author Valery Fremaux (France) (admin@www.ethnoinformatique.fr)
    * @date 2008/03/03
    * @version phase1
    * @contributors LUU Tao Meng, So Gerard (parts of treelib.php), Guillaume Magnien, Olivier Petit
    * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
    */

    $defaultformat = FORMAT_MOODLE;

    /**
    * a form constraint checking function
    * @param object $project the surrounding project cntext
    * @param object $task form object to be checked
    * @return a control hash array telling error statuses
    */
    function checkConstraints($project, $task){
        $control = NULL;
    
        switch($project->timeunit){
            case HOURS : $plannedtime = 3600 ; break ;
            case HALFDAY : $plannedtime = 3600 * 12 ; break ;
            case DAY : $plannedtime = 3600 * 24 ; break ;
            default : $plannedtime = 0;
        }
        // checking too soon task
        if ($task->taskstartenable && $task->taskstart < $project->projectstart){
            $control['taskstartdate'] = get_string('tasktoosoon','techproject') . '<br/>' . userdate($project->projectstart);
        }
        // task too late (planned to milestone)
        if($task->taskstartenable && $task->milestoneid){
            $milestone = get_record('techproject_milestone', 'projectId', $project->id, 'id', $task->milestoneid);
            if ($milestone->deadlineenable && ($task->taskstart + $plannedtime > $milestone->deadline)){
                $control['taskstartdate'] = get_string('taskstartsaftermilestone','techproject') . '<br/>' . userdate($milestone->deadline);
            }
        }
        // task too late (absolute)
        elseif($task->taskstartenable && ($task->taskstart + $plannedtime > $project->projectend)){
            $control['taskstartdate'] = get_string('tasktoolate','techproject') . '<br/>' . userdate($project->projectend);
        }
        // checking too late end
        elseif($task->taskendenable && $task->milestoneid){
            $milestone = get_record('techproject_milestone', 'projectId', $project->id, 'id', $task->milestoneid);
            if ($milestone->deadlineenable && ($task->taskend > $milestone->deadline)){
                $control['taskenddate'] = get_string('taskfinishesaftermilestone','techproject') . '<br/>' . userdate($milestone->deadline);
            }
        }
        // checking too late end
        elseif($task->taskendenable && $task->taskend > $project->projectend){
            $control['taskenddate'] = get_string('taskfinishestoolate','techproject') . '<br/>' . userdate($project->projectend);
        }
        // checking switched end and start
        elseif($task->taskendenable && $task->taskstartenable && $task->taskend <= $task->taskstart){
            $control['taskenddate'] = get_string('taskfinishesbeforeitstarts','techproject');
        }
        // checking unfeseabletask
        elseif($task->taskendenable && $task->taskstartenable && $task->taskend < $task->taskstart + $plannedtime){
            $control['taskenddate'] = get_string('tasktooshort','techproject') . '<br/> >> ' . userdate($task->taskstart + $plannedtime);
        }
        
        return $control;
    }

/// Controller
    
    $usehtmleditor = can_use_html_editor();
    $TIMEUNITS = array(get_string('unset','techproject'),get_string('hours','techproject'),get_string('halfdays','techproject'),get_string('days','techproject'));
    
   	if ($work == 'new') {
   		$task->groupid = $currentGroupId;
   		$task->projectid = $project->id;
        $task->abstract = required_param('abstract', PARAM_TEXT);
        $task->description = addslashes(required_param('description', PARAM_CLEANHTML));
        $task->format = required_param('format', PARAM_INT);
        $task->fatherid = required_param('fatherid', PARAM_INT);
        $task->worktype = required_param('worktype', PARAM_ALPHA);
        $task->assignee = optional_param('assignee', 0, PARAM_INT);
        $task->owner = required_param('owner', PARAM_INT);
        $task->status = required_param('status', PARAM_ALPHA);
        $task->costrate = optional_param('costrate', '0', PARAM_INT);
        $task->planned = 0 + required_param('planned', PARAM_INT);
        $task->cost = 0 + required_param('cost', PARAM_INT);
        $task->done = 0 + required_param('done', PARAM_INT);
        $task->milestoneid = optional_param('milestoneid', 0, PARAM_INT);
        $task->taskstartenable = optional_param('taskstartenable', '0', PARAM_INT);
        $taskstartyear = optional_param('taskstartyear', 0, PARAM_INT);
        $taskstartmonth = optional_param('taskstartmonth', 0, PARAM_INT);
        $taskstartday = optional_param('taskstartday', 0, PARAM_INT);
        $taskstarthour = optional_param('taskstarthour', 0, PARAM_INT);
        $taskstartminute = optional_param('taskstartminute', 0, PARAM_INT);
        $task->taskstart = 0 + make_timestamp($taskstartyear, 
            $taskstartmonth, $taskstartday, $taskstarthour, 
            $taskstartminute);
        if ($task->taskstart == 0) $task->taskstart = time();
        $taskendyear = optional_param('taskendyear', 0, PARAM_INT);
        $taskendmonth = optional_param('taskendmonth', 0, PARAM_INT);
        $taskendday = optional_param('taskendday', 0, PARAM_INT);
        $taskendhour = optional_param('taskendhour', 0, PARAM_INT);
        $taskendminute = optional_param('taskendminute', 0, PARAM_INT);
        $task->taskendenable = optional_param('taskendenable', '0', PARAM_INT);
        $task->taskend = 0 + make_timestamp($taskendyear, 
           $taskendmonth, $taskendday, $taskendhour, 
           $taskendminute);
        if ($task->taskend == 0) $task->taskend = $project->projectend;
   		$task->userid = $USER->id;
   		$task->created = time();
   		$task->modified = time();
   		$task->lastuserid = $USER->id;
           
           // perform insertion
           $controls = checkConstraints($project, $task);
           if (!$controls){
               if (!empty($task->abstract)){
               	$lastordering = techproject_tree_get_max_ordering($project->id, $currentGroupId, 'techproject_task', true, $task->fatherid);
               	$task->ordering = $lastordering + 1;
           		$returnid = insert_record('techproject_task', $task);
                   add_to_log($course->id, 'techproject', 'changetask', "view.php?id={$cm->id}&amp;view=tasks&amp;group={$currentGroupId}", 'add', $cm->id);
           		
           		// if subtask, force dependency on father
           		if ($task->fatherid != 0){
           		    $aDependency->id = 0;
           		    $aDependency->projectId = $project->id;
           		    $aDependency->groupId = $currentGroupId;
           		    $aDependency->slave = $task->fatherid;
           		    $aDependency->master = $returnid;
           		    insert_record('techproject_task_dependency', $aDependency);
           		}
           		
           		// if subtask, calculate branch propagation
           		if ($task->fatherid != 0){
           		    techproject_tree_propagate_up('techproject_task', 'done', $returnid, '~');
           		    techproject_tree_propagate_up('techproject_task', 'planned', $returnid, '+');
           		    techproject_tree_propagate_up('techproject_task', 'cost', $returnid, '+');
           		}
           	}
           } else {
              $work = 'add';
           }
   	} elseif ($work == 'doupdate') {
   		$task->id = required_param('taskid', PARAM_INT);
   		$task->fatherid = required_param('fatherid', PARAM_INT);
   		$task->abstract = required_param('abstract', PARAM_TEXT);
   		$task->description = addslashes(required_param('description', PARAM_CLEANHTML));
        $task->format = required_param('format', PARAM_INT);
   		$task->assignee = optional_param('assignee', 0, PARAM_INT);
   		$task->owner = required_param('owner', PARAM_INT);
   		$task->done = required_param('done', PARAM_INT);
   		$task->status = required_param('status', PARAM_ALPHA);
   		$task->worktype = required_param('worktype', PARAM_ALPHA);
   		$task->planned = required_param('planned', PARAM_INT);
   		$task->cost = required_param('cost', PARAM_INT);
   		$task->costrate = required_param('costrate', PARAM_INT);
        $task->milestoneid = optional_param('milestoneid', 0, PARAM_INT);
        $task->taskstartenable = optional_param('taskstartenable', '0', PARAM_INT);
        $taskstartyear = optional_param('taskstartyear', 0, PARAM_INT);
        $taskstartmonth = optional_param('taskstartmonth', 0, PARAM_INT);
        $taskstartday = optional_param('taskstartday', 0, PARAM_INT);
        $taskstarthour = optional_param('taskstarthour', 0, PARAM_INT);
        $taskstartminute = optional_param('taskstartminute', 0, PARAM_INT);
        $task->taskstart = 0 + make_timestamp($taskstartyear, 
               $taskstartmonth, $taskstartday, $taskstarthour, 
               $taskstartminute);
        if ($task->taskstart == 0) unset($task->taskstart);
        $task->taskendenable = optional_param('taskendenable', '0', PARAM_INT);
        $taskendyear = optional_param('taskendyear', 0, PARAM_INT);
        $taskendmonth = optional_param('taskendmonth', 0, PARAM_INT);
        $taskendday = optional_param('taskendday', 0, PARAM_INT);
        $taskendhour = optional_param('taskendhour', 0, PARAM_INT);
        $taskendminute = optional_param('taskendminute', 0, PARAM_INT);
        $task->taskend = 0 + make_timestamp($taskendyear, 
           $taskendmonth, $taskendday, $taskendhour, 
           $taskendminute);
        if ($task->taskend == 0) unset($task->taskend);
   		$task->modified = time();
   		$task->lastuserid = $USER->id;
   
           $controls = checkConstraints($project, $task);
           if (!$controls){
               if (!empty($task->abstract)){
           		$oldAssigneeId = get_field('techproject_task', 'assignee', 'id', $task->id);
       		    $res = update_record('techproject_task', $task);
                   add_to_log($course->id, 'techproject', 'changetask', "view.php?id={$cm->id}&amp;view=tasks&amp;group={$currentGroupId}", 'update', $cm->id);
       
           		// storing mapping task to spec
           		$tasktospec = optional_param('tasktospec', null, PARAM_INT);
           		if (count($tasktospec) > 0){
           		    // removes previous mapping
           		    delete_records('techproject_task_to_spec', 'projectId', $project->id, 'groupId', $currentGroupId, 'taskId', $task->id);
           
           		    // stores new mapping
               		foreach($tasktospec as $aSpec){
               		    $amap->id = 0;
               		    $amap->projectId = $project->id;
               		    $amap->groupId = $currentGroupId;
               		    $amap->specId = $aSpec;
               		    $amap->taskId = $task->id;
               		    $res = insert_record('techproject_task_to_spec', $amap);
               		}
               	}
           
           		// storing mapping task to task (dependencies)
           		$taskdependency = optional_param('taskdependency', null, PARAM_INT);
           		if (count($taskdependency) > 0){
           		    // removes previous mapping
           		    delete_records('techproject_task_dependency', 'projectId', $project->id, 'groupId', $currentGroupId, 'slave', $task->id);
           
           		    // stores new mapping
               		// if subtask, force dependency on father
               		if ($task->fatherid != 0){
               		    $aDependancy->id = 0;
               		    $aDependancy->projectId = $project->id;
               		    $aDependancy->groupId = $currentGroupId;
               		    $aDependancy->slave = $task->id;
               		    $aDependancy->master = $task->fatherid;
               		    insert_record('techproject_task_to_task', $aDependancy);
                       }
                       // store other mappings
               		foreach($taskdependency as $aMaster){
               		    $amap->id = 0;
               		    $amap->projectId = $project->id;
               		    $amap->groupId = $currentGroupId;
              		        $amap->master = $aMaster;
               		    $amap->slave = $task->id;
               		    $res = insert_record('techproject_task_dependency', $amap);
               		}
               	}
           
           		// storing mapping task to deliv
           		$tasktodeliv = optional_param('tasktodeliv', null, PARAM_INT);
           		if (count($tasktodeliv) > 0){
           		    // removes previous mapping
           		    delete_records('techproject_task_to_deliv', 'projectId', $project->id, 'groupId', $currentGroupId, 'taskId', $task->id);
           
           		    // stores new mapping
               		foreach($tasktodeliv as $aDeliv){
               		    $amap->id = 0;
               		    $amap->projectId = $project->id;
               		    $amap->groupId = $currentGroupId;
               		    $amap->delivId = $aDeliv;
               		    $amap->taskId = $task->id;
               		    $res = insert_record('techproject_task_to_deliv', $amap);
               		}
               	}
           
           		// if subtask, calculate branch propagation
           		if ($task->fatherid != 0){
           		    techproject_tree_propagate_up('techproject_task', 'done', $task->id, '~', false);
           		    techproject_tree_propagate_up('techproject_task', 'planned', $task->id, '+', false);
           		    techproject_tree_propagate_up('techproject_task', 'cost', $task->id, '+', false);
           		}
           		
           		// if notifications allowed and previous assignee exists (and is not the new assignee) notify previous assignee
           		if( $project->allownotifications && !empty($oldAssigneeId) && $task->assignee != $oldAssigneeId){
               		$oldAssignee = get_record('user', 'id', $oldAssigneeId);
               		$owner = get_record('user', 'id', $task->owner);
               		$worktype = get_record('techproject_worktype', 'worktype', $task->worktype);
               		$projectheading = get_record('techproject_heading', 'projectid', $project->id, 'groupid', $currentGroupId);
               		$message = compile_mail_template('taskreleasenotify', array(
               		    'PROJECT' => $projectheading->title,
               		    'OWNER' => fullname($owner),
               		    'TASKNODE' => implode(".", techproject_tree_get_upper_branch('techproject_task', $task->id, true, true)),
               		    'TASKABSTRACT' => stripslashes($task->abstract),
               		    'TASKDESCRIPTION' => $task->description,
               		    'WORKTYPE' => $worktype->label,
               		    'DONE' => $task->done
               		), 'techproject');
               		email_to_user ($oldAssignee, $owner, $course->shortname .' - '.get_string('notifyreleasedtask', 'techproject'), html_to_text($message), $message);
               	}
           
           		// if notifications allowed and assignee set notify assignee
           		if( $project->allownotifications && !empty($task->assignee)){
               		$assignee = get_record('user', 'id', $task->assignee);
               		$owner = get_record('user', 'id', $task->owner);
               		$worktype = get_record('techproject_worktype', 'worktype', $task->worktype);
               		$projectheading = get_record('techproject_heading', 'projectid', $project->id, 'groupid', $currentGroupId);
               		$message = compile_mail_template('newtasknotify', array(
               		    'PROJECT' => $projectheading->title,
               		    'OWNER' => fullname($owner),
               		    'TASKNODE' => implode(".", techproject_tree_get_upper_branch('techproject_task', $task->id, true, true)),
               		    'TASKABSTRACT' => stripslashes($task->abstract),
               		    'TASKDESCRIPTION' => $task->description,
               		    'WORKTYPE' => $worktype->label,
               		    'DONE' => $task->done
               		), 'techproject');
               		email_to_user ($assignee, $owner, $course->shortname .' - '.get_string('notifynewtask', 'techproject'), html_to_text($message), $message);
               	}
               }
           } else {
               $work = 'update';
           }
   	} elseif ($work == 'dodelete') {
        $taskid = required_param('taskid', PARAM_INT);
   	    
   	    // save record for further cleanups
   	    $oldtask = get_record('techproject_task', 'id', $taskid);
   
        // delete all related records
   		techproject_tree_delete($taskid, 'techproject_task');
   
        add_to_log($course->id, 'techproject', 'changetask', "view.php?id={$cm->id}&amp;view=tasks&amp;group={$currentGroupId}", 'delete', $cm->id);
   
        //reset indicators 
        $oldtask->done = 0;
        $oldtask->planned = 0;
        $oldtask->cost = 0;
        update_record('techproject_task', $oldtask);
   
   		// if was subtask, update branch annulation
   		if ($oldtask->fatherid != 0){
   		    techproject_tree_propagate_up('techproject_task', 'done', $oldtask->id, '~');
   		    techproject_tree_propagate_up('techproject_task', 'planned', $oldtask->id, '+');
   		    techproject_tree_propagate_up('techproject_task', 'cost', $oldtask->id, '+');
   		}
   
           // now can delete records
   		delete_records('techproject_task_to_spec', 'projectId', $project->id, 'groupId', $currentGroupId, 'taskId', $taskid);
   		delete_records('techproject_task_dependency', 'projectId', $project->id, 'groupId', $currentGroupId, 'master', $taskid);
   		delete_records('techproject_task_dependency', 'projectId', $project->id, 'groupId', $currentGroupId, 'slave', $taskid);   
   	} elseif ($work == 'domarkasdone') {    	
        // just completes a task with 100% done indicator.
   		$ids = required_param('ids', PARAM_INT);
   		if (is_array($ids)){
       		foreach($ids as $anItem){
       		    unset($object);
       		    $object->id = $anItem;
       		    $object->done = 100;
                   update_record('techproject_task', $object);
       		}
       	}
   	}
   	// full fills a task with planned values and 100% done indicator.
   	elseif ($work == 'fullfill') {
   		$ids = required_param('ids', PARAM_INT);
   		if (is_array($ids)){
   		    $task = get_record('techproject_task', 'id', $anItem);
       		foreach($ids as $anItem){
       		    unset($object);
       		    $object->id = $task->id;
       		    $object->done = 100;
       		    $object->cost = $task->planned;
                   update_record('techproject_task', $object);
       		}
       	}
   	}
   	elseif ($work == 'domove' || $work == 'docopy') {
   		$ids = required_param('ids', PARAM_INT);
   		$to = required_param('to', PARAM_ALPHA);
   		switch($to){
   		    case 'requs' : { $table2 = 'techproject_requirement'; $redir = 'requirement'; } break;
   		    case 'specs' : { $table2 = 'techproject_specification'; $redir = 'specification'; } break;
   		    case 'tasks' : { $table2 = 'techproject_task'; $redir = 'task'; } break;
   		    case 'deliv' : { $table2 = 'techproject_deliverable'; $redir = 'deliverable'; } break;
   		}
   		techproject_tree_copy_set($ids, 'techproject_task', $table2);
           add_to_log($course->id, 'techproject', 'change{$redir}', "view.php?id={$cm->id}&amp;view={$redir}s&amp;group={$currentGroupId}", 'copy/move', $cm->id);
   		if ($work == 'domove'){
   		    // bounce to deleteitems
   		    $work = 'dodeleteitems';
   		    $withredirect = 1;
   		} else {
   		    redirect("{$CFG->wwwroot}/mod/techproject/view.php?id={$cm->id}&amp;view={$redir}s", get_string('redirectingtoview', 'techproject') . ' : ' . get_string($redir, 'techproject'));
   	    }
   	}
   	if ($work == 'dodeleteitems') {
   		$ids = required_param('ids', PARAM_INT);
   		foreach($ids as $anItem){		    
       	    // save record for further cleanups and propagation
       	    $oldtask = get_record('techproject_task', 'id', $anItem);
   		    $childs = get_records('techproject_task', 'fatherid', $anItem);
   		    
   		    // update fatherid in childs 
   		    $query = "
   		        UPDATE
   		            {$CFG->prefix}techproject_task
   		        SET
   		            fatherid = $oldtask->fatherid
   		        WHERE
   		            fatherid = $anItem
   		    ";
   		    execute_sql($query);
   
               //reset indicators 
               $oldtask->done = 0;
               $oldtask->planned = 0;
               $oldtask->cost = 0;
               update_record($oldtask);
       
       		// if was subtask, update branch propagation
       		if ($oldtask->fatherid != 0){
       		    techproject_tree_propagate_up('techproject_task', 'done', $oldtask->id, '~');
       		    techproject_tree_propagate_up('techproject_task', 'planned', $oldtask->id, '+');
       		    techproject_tree_propagate_up('techproject_task', 'cost', $oldtask->id, '+');
       		}
   
               // delete record for this item
       		delete_records('techproject_task', 'id', $anItem);
            add_to_log($course->id, 'techproject', 'changetask', "view.php?id={$cm->id}&amp;view=tasks&amp;group={$currentGroupId}", 'deleteItems', $cm->id);

               // delete all related records
       		delete_records('techproject_task_to_spec', 'projectId', $project->id, 'groupId', $currentGroupId, 'taskId', $anItem);
       		delete_records('techproject_task_dependency', 'projectId', $project->id, 'groupId', $currentGroupId, 'master', $anItem);
       		delete_records('techproject_task_dependency', 'projectId', $project->id, 'groupId', $currentGroupId, 'slave', $anItem);
       		
       		// must rebind child dependencies to father 
       		if ($oldtask->fatherid != 0 && $childs){
           		foreach($childs as $aChild){
           		    $aDependency->id = 0;
           		    $aDependency->projectId = $project->id;
           		    $aDependency->groupId = $currentGroupId;
           		    $aDependency->master = $oldtask->fatherid;
           		    $aDependency->slave = $aChild->id;
           		    insert_record('techproject_task_dependency', $aDependency);
           		}
           	}   
       	}
       	if (isset($withredirect) && $withredirect){
   		    redirect("{$CFG->wwwroot}/mod/techproject/view.php?id={$cm->id}&amp;view={$redir}s", get_string('redirectingtoview', 'techproject') . ' : ' . get_string($redir, 'techproject'));
   		}
   	} elseif ($work == 'doclearall') {
           // delete all related records. POWERFUL AND DANGEROUS COMMAND.
           // deletes for the current group. 
   		delete_records('techproject_task', 'projectId', $project->id, 'groupId', $currentGroupId);
   		delete_records('techproject_task_to_spec', 'projectId', $project->id, 'groupId', $currentGroupId);
   		delete_records('techproject_task_to_deliv', 'projectId', $project->id, 'groupId', $currentGroupId);
   		delete_records('techproject_task_dependency', 'projectId', $project->id, 'groupId', $currentGroupId);
           add_to_log($course->id, 'techproject', 'changetask', "view.php?id={$cm->id}&amp;view=tasks&amp;group={$currentGroupId}", 'clear', $cm->id);
   	} elseif ($work == 'doexport') {
   	    $ids = required_param('ids', PARAM_INT);
   	    $idlist = implode("','", $ids);
   	    $select = "
   	       id IN ('$idlist')	       
   	    ";
   	    $tasks = get_records_select('techproject_task', $select);
   	    $worktypes = get_records('techproject_worktype', 'projectid', $project->id);
   	    if (empty($worktypes)){
   	        $worktypes = get_records('techproject_worktype', 'projectid', 0);
   	    }
   	    $taskstatusses = get_records('techproject_taskstatus', 'projectid', $project->id);
   	    if (empty($taskstatusses)){
   	        $taskstatusses = get_records('techproject_taskstatus', 'projectid', 0);
   	    }
   	    include "xmllib.php";
   	    $xmlworktypes = recordstoxml($worktypes, 'worktype_option', '', false, 'techproject');
   	    $xmltaskstatusses = recordstoxml($taskstatusses, 'task_status_option', '', false, 'techproject');
   	    $xml = recordstoxml($tasks, 'task', $xmlworktypes.$xmltaskstatusses, true, null);
   	    $escaped = str_replace('<', '&lt;', $xml);
   	    $escaped = str_replace('>', '&gt;', $escaped);
   	    print_heading(get_string('xmlexport', 'techproject'));
   	    print_simple_box("<pre>$escaped</pre>");
           add_to_log($course->id, 'techproject', 'readtask', "view.php?id={$cm->id}&amp;view=tasks&amp;group={$currentGroupId}", 'export', $cm->id);
           print_continue("view.php?view=tasks&amp;id=$cm->id");
           return;
   	} elseif ($work == 'up') {
   	    $taskid = required_param('taskid', PARAM_INT);
   		techproject_tree_up($project, $currentGroupId, $taskid, 'techproject_task');
   	} elseif ($work == 'down') {
   	    $taskid = required_param('taskid', PARAM_INT);
   		techproject_tree_down($project, $currentGroupId, $taskid, 'techproject_task');
   	} elseif ($work == 'left') {
   	    $taskid = required_param('taskid', PARAM_INT);
   		techproject_tree_left($project, $currentGroupId, $taskid, 'techproject_task');
   	    techproject_tree_propagate_up('techproject_task', 'done', $taskid, '~');
   	    techproject_tree_propagate_up('techproject_task', 'planned', $taskid, '+');
   	    techproject_tree_propagate_up('techproject_task', 'cost', $taskid, '+');
   	} elseif ($work == 'right') {
   	    $taskid = required_param('taskid', PARAM_INT);
   		techproject_tree_right($project, $currentGroupId, $taskid, 'techproject_task');
   	}
    
/// Add task form *********************************************************

   	if ($work == 'add'){
   	    $fatherid = required_param('fatherid', PARAM_INT);
   	    $tasktitle = ($fatherid) ? 'addsubtask' : 'addtask';
    ?>
    
    <center>
    <?php print_heading(get_string($tasktitle, 'techproject'), 'center'); ?>
    <script type="text/javascript">
    //<![CDATA[
    function senddata(){
        document.forms['addtaskform'].work.value='new';
        <?php if ($usehtmleditor) echo "document.forms['addtaskform'].onsubmit();\n"; ?>
        document.forms['addtaskform'].submit();
    }
    
    function cancel(){
        <?php if ($usehtmleditor) echo "document.forms['addtaskform'].onsubmit();\n"; ?>
        document.forms['addtaskform'].submit();
    }
    
    var taskstartitems = ['taskstartday','taskstartmonth','taskstartyear','taskstarthour', 'taskstartminute'];
    var taskenditems = ['taskendday','taskendmonth','taskendyear','taskendhour', 'taskendminute'];
    //]]>
    </script>
    <form name="addtaskform" method="post" action="view.php#tsk<?php p($fatherid) ?>">
    <input type="hidden" name="id" value="<?php p($cm->id) ?>" />
    <input type="hidden" name="fatherid" value="<?php echo $fatherid; ?>" />
    <input type="hidden" name="work" value="" />
    <table>
    <tr valign="top">
    	<td align="right"><b><?php print_string('tasktitle', 'techproject') ?>:</b></td>
        <td align="left">
            <input type="text" name="abstract" size="100%" value="<?php p(@$task->abstract) ?>" alt="<?php  print_string('tasktitle', 'techproject') ?>" />
        </td>
    </tr>
    <tr valign="top">
    	<td align="right"><b><?php print_string('owner', 'techproject') ?>:</b></td>
        <td align="left">
            <?php
                echo $USER->lastname . " " . $USER->firstname . " ";
                print_user_picture($USER->id, $course->id, !empty($USER->picture)); 
            ?>
            <input type="hidden" name="owner" value="<?php echo p($USER->id); ?>" />
        </td>
    </tr>
    <tr valign="top">
    	<td align="right"><b><?php print_string('assignee', 'techproject') ?>:</b></td>
        <td align="left">
    <?php 
            $assignees = techproject_get_group_users($project->course, $cm, $currentGroupId);
            if($assignees){
                $assignoptions = array();
                foreach($assignees as $anAssignee){
                    $assignoptions[$anAssignee->id] = $anAssignee->lastname . ' ' . $anAssignee->firstname;
                }
                choose_from_menu($assignoptions, 'assignee', 0, get_string('unassigned','techproject'));
            }
            else{
               print_string('noassignee', 'techproject');
            }
     ?>
        </td>
    </tr>
    
    <tr valign="top">
        <td align="right"><b><?php print_string('taskstartdate', 'techproject') ?>:
                <?php if (isset($controls['taskstartdate'])) echo "<br/><span class=\"inconsistency\">{$controls['taskstartdate']}</span>" ?></b></td>
        <td align="left">
                <input name="taskstartenable" type="checkbox" value="1" alt="<?php print_string('taskstartenable', 'techproject') ?>" onclick="return lockoptions('addtaskform', 'taskstartenable', taskstartitems)" />
            <?php
                print_date_selector('taskstartday', 'taskstartmonth', 'taskstartyear', time());
                echo "&nbsp;-&nbsp;";
                print_time_selector('taskstarthour', 'taskstartminute', time());
                helpbutton('taskstartdate', get_string('taskstartdate', 'techproject'), 'techproject', true, false);
        ?>
            <input type="hidden" name="htaskstartday"    value="0" />
            <input type="hidden" name="htaskstartmonth"  value="0" />
            <input type="hidden" name="htaskstartyear"   value="0" />
            <input type="hidden" name="htaskstarthour"   value="0" />
            <input type="hidden" name="htaskstartminute" value="0" />
        </td>
    </tr>
    
    <tr valign="top">
        <td align="right"><b><?php print_string('taskenddate', 'techproject') ?>:
                <?php if (isset($controls['taskenddate'])) echo "<br/><span class=\"inconsistency\">{$controls['taskenddate']}</span>" ?></b></td>
        <td align="left">
                <input name="taskendenable" type="checkbox" value="1" alt="<?php print_string('taskendenable', 'techproject') ?>" onclick="return lockoptions('addtaskform', 'taskendenable', taskenditems)" />
            <?php
                print_date_selector('taskendday', 'taskendmonth', 'taskendyear', time());
                echo "&nbsp;-&nbsp;";
                print_time_selector('taskendhour', 'taskendminute', time());
                helpbutton('taskenddate', get_string('taskenddate', 'techproject'), 'techproject', true, false);
        ?>
            <input type="hidden" name="htaskendday"    value="0" />
            <input type="hidden" name="htaskendmonth"  value="0" />
            <input type="hidden" name="htaskendyear"   value="0" />
            <input type="hidden" name="htaskendhour"   value="0" />
            <input type="hidden" name="htaskendminute" value="0" />
        </td>
    </tr>
    
    <tr valign="top">
    	<td align="right"><b><?php print_string('worktype', 'techproject') ?>:</b>
    	</td>
        <td align="left">
    <?php
            $worktypes = techproject_get_options('techproject_worktype', $project->id);
            $worktypeoptions = array();
            foreach($worktypes as $aWorktype){
                $worktypeoptions[$aWorktype->worktype] = '['. $aWorktype->worktype . '] ' . $aWorktype->label;
            }
            choose_from_menu ($worktypeoptions, 'worktype', @$task->worktype, 'choose', '', '---');
            helpbutton('worktype', get_string('worktype', 'techproject'), 'techproject', true, false);
    ?>
        </td>
    </tr>
    <tr valign="top">
    	<td align="right"><b><?php  print_string('status', 'techproject') ?>:</b>
    	</td>
        <td align="left">
    <?php
            $statusses = techproject_get_options('techproject_taskstatus', $project->id);
            $statussesoptions = array();
            foreach($statusses as $aStatus){
                $statussesoptions[$aStatus->taskstatus] = '['. $aStatus->taskstatus . '] ' . $aStatus->label;
            }
            choose_from_menu ($statussesoptions, 'status', @$task->status, 'choose', '', 'PLANNED');
            helpbutton('status', get_string('status', 'techproject'), 'techproject', true, false);
    ?>
        </td>
    </tr>
    
    <tr valign="top">
    	<td align="right"><b><?php print_string('costrate', 'techproject') ?>:</b></td>
        <td align="left">
            <input type="text" name="costrate" size="3" value="" alt="<?php print_string('costrate', 'techproject') ?>" />
        </td>
    </tr>
    <tr valign="top">
    	<td align="right"><b><?php print_string('cost', 'techproject') ?>:</b></td>
        <td align="left">
            <input type="text" name="planned" size="3" value="<?php p(@$task->planned) ?>" alt="<?php print_string('cost', 'techproject') ?>" /><?php echo $TIMEUNITS[$project->timeunit] ?>
            <?php helpbutton('costplanned', get_string('cost', 'techproject'), 'techproject', true, false); ?>
        </td>
    </tr>
    <tr valign="top">
    	<td align="right"><b><?php  print_string('done', 'techproject') ?>:</b></td>
        <td align="left">
            <input type="text" name="done" size="3" value="<?php p(@$task->done) ?>" alt="<?php print_string('done', 'techproject') ?>" /> %
            <?php helpbutton('done', get_string('done', 'techproject'), 'techproject', true, false); ?>
        </td>
    </tr>
    <tr valign="top">
    	<td align="right"><b><?php  print_string('costelapsed', 'techproject') ?>:</b></td>
        <td align="left">
            <input type="text" name="cost" size="3" value="<?php p(@$task->costelapsed) ?>" alt="<?php print_string('cost', 'techproject') ?>" /><?php echo $TIMEUNITS[$project->timeunit] ?>
        </td>
    </tr>
    <tr valign="top">
        <td align="right"><b><?php print_string('description', 'techproject') ?>:</b><br />
        <font size="1">
    <?php
            helpbutton('writing', get_string('helpwriting'), 'moodle', true, true);
            echo "<br />";
            if ($usehtmleditor) {
               helpbutton('richtext', get_string('helprichtext'), 'moodle', true, true);
            } else {
               helpbutton('text', get_string('helptext'), 'moodle', true, true);
               echo "<br />";
               emoticonhelpbutton('addtaskform', 'description', 'moodle', true, true);
               echo "<br />";
            }
    ?>
          <br />
        </font>
        </td>
        <td align="right">
    <?php
           print_textarea($usehtmleditor, 20, 60, 595, 400, 'description', @$task->description);
    
           if ($usehtmleditor) {
               echo '<input type="hidden" name="format" value="'.FORMAT_HTML.'" />';
               $nohtmleditorneeded = false;
           } else {
               echo '<p align="right">';
               helpbutton('textformat', get_string('formattexttype'));
               print_string('formattexttype');
               echo ':&nbsp;';
               choose_from_menu(format_text_menu(), 'format', $defaultformat, '');
               echo '</p>';
           }
    ?>
        </td>
    </tr>
    <tr valign="top">
    	<td align="right"><b><?php print_string('milestone', 'techproject') ?>:</b>
    	</td>
        <td align="left">
    <?php
        $milestones = get_records_select('techproject_milestone', "projectId = {$project->id} AND groupId = {$currentGroupId}", 'ordering ASC', 'id, abstract, ordering');
        $milestonesoptions = array();
        if ($milestones){
            foreach($milestones as $aMilestone){
                $milestonesoptions[$aMilestone->id] = $aMilestone->abstract;
            }
            choose_from_menu ($milestonesoptions, 'milestoneid', 0, get_string('unassigned', 'techproject') );
        }
        else{
            print_string('nomilestones', 'techproject');
        }
        helpbutton('tasktomilestone', get_string('milestone', 'techproject'), 'techproject', true, false);
    ?>
        </td>
    </tr>
    </table>
    <input type="button" name="go_btn" value="<?php print_string('savechanges'); ?>" onclick="senddata()" />
    <input type="button" name="cancel_btn" value="<?php print_string('cancel'); ?>" onclick="cancel()" />
    </form>
    <script type="text/javascript">
    //<![CDATA[
    lockoptions('addtaskform', 'taskstartenable', taskstartitems);
    lockoptions('addtaskform', 'taskendenable', taskenditems);
    //]]>
    </script>
    </center>
    
    <?php
    	}
    
/// Update task form *********************************************************

    	elseif ($work == 'update') {
    	    $taskid = required_param('taskid', PARAM_INT);
    
            // if not a bounce from doupdate get the old record
            if (!isset($task)) $task = get_record('techproject_task', 'id', $taskid);
    		if(!$task){
    			error(print_string('errortask','techproject'));
    		}
    		else {
    ?>
    
    <center>
    <?php print_heading(get_string('updatetask', 'techproject'), 'center'); ?>
    <script type="text/javascript">
    //<![CDATA[
    function senddata(){
        document.forms['updatetaskform'].work.value='doupdate';
        <?php if ($usehtmleditor) echo "document.forms['updatetaskform'].onsubmit();\n"; ?>
        document.forms['updatetaskform'].submit();
    }
    
    function cancel(){
        <?php if ($usehtmleditor) echo "document.forms['updatetaskform'].onsubmit();\n"; ?>
        document.forms['updatetaskform'].submit();
    }
    
    var taskstartitems = ['taskstartday','taskstartmonth','taskstartyear','taskstarthour', 'taskstartminute'];
    var taskenditems = ['taskendday','taskendmonth','taskendyear','taskendhour', 'taskendminute'];
    //]]>
    </script>
    <form name="updatetaskform" method="post" action="view.php#tsk<?php p($task->fatherid) ?>">
    <input type="hidden" name="id" value="<?php p($cm->id) ?>" />
    <input type="hidden" name="fatherid" value="<?php echo $task->fatherid; ?>" />
    <input type="hidden" name="work" value="" />
    <input type="hidden" name="taskid" value="<?php p($task->id) ?>" />
    <table>
    <tr valign="top">
    	<td align="right"><b><?php print_string('tasktitle', 'techproject') ?>:</b></td>
        <td align="left">
            <input type="text" name="abstract" size="100%" value="<?php p($task->abstract) ?>" alt="<?php print_string('tasktitle', 'techproject') ?>" />
        </td>
    </tr>
    <tr valign="top">
    	<td align="right"><b><?php print_string('owner', 'techproject') ?>:</b></td>
        <td align="left">
    <?php 
            $groupusers = techproject_get_group_users($project->course, $cm, $currentGroupId);
            $owneroptions = array();
            foreach($groupusers as $anOwner){
                $owneroptions[$anOwner->id] = fullname($anOwner);
            }
            choose_from_menu ($owneroptions, 'owner', $task->owner, get_string('unassigned', 'techproject'));
     ?>
        </td>
    </tr>
    <tr valign="top">
    	<td align="right"><b><?php print_string('assignee', 'techproject') ?>:</b></td>
        <td align="left">
    <?php 
            if($groupusers){
                $assignoptions = array();
                foreach($groupusers as $anAssignee){
                    $assignoptions[$anAssignee->id] = fullname($anAssignee);
                }
                choose_from_menu ($assignoptions, 'assignee', $task->assignee, get_string('unassigned','techproject'));
            }
            else{
               print_string('noassignee', 'techproject');
            }
     ?>
            <input type="hidden" name="oldassignee" value="$task->assignee" />
        </td>
    </tr>
    
    <tr valign="top">
        <td align="right" class="<?php if (isset($controls['taskstartdate'])) echo 'inconsistency'; ?>"><b><?php print_string('taskstartdate', 'techproject') ?>:
                <?php if (isset($controls['taskstartdate'])) echo "<br/><span class=\"inconsistency\">{$controls['taskstartdate']}</span>" ?></b></td>
        <td align="left">
                <input name="taskstartenable" type="checkbox" value="1" alt="<?php print_string('taskstartenable', 'techproject') ?>" onclick="return lockoptions('updatetaskform', 'taskstartenable', taskstartitems)" <?php if ($task->taskstartenable) echo 'checked="checked"' ?> />
            <?php
                if ($task->taskstart == -1) $task->taskstart = time();
                print_date_selector('taskstartday', 'taskstartmonth', 'taskstartyear', $task->taskstart);
                echo "&nbsp;-&nbsp;";
                print_time_selector('taskstarthour', 'taskstartminute', $task->taskstart);
                helpbutton('taskstartdate', get_string('taskstartdate', 'techproject'), 'techproject', true, false);
        ?>
            <input type="hidden" name="htaskstartday"    value="0" />
            <input type="hidden" name="htaskstartmonth"  value="0" />
            <input type="hidden" name="htaskstartyear"   value="0" />
            <input type="hidden" name="htaskstarthour"   value="0" />
            <input type="hidden" name="htaskstartminute" value="0" />
        </td>
    </tr>
    
    <tr valign="top">
        <td align="right" class="<?php if (isset($controls['taskenddate'])) echo 'inconsistency'; ?>"><b><?php print_string('taskenddate', 'techproject') ?>:
                <?php if (isset($controls['taskenddate'])) echo "<br/><span class=\"inconsistency\">{$controls['taskenddate']}</span>" ?></b></td>
        <td align="left">
                <input name="taskendenable" type="checkbox" value="1" alt="<?php print_string('taskendenable', 'techproject') ?>" onclick="return lockoptions('updatetaskform', 'taskendenable', taskenditems)" <?php if ($task->taskendenable) echo 'checked="checked"' ?> />
            <?php
                // fix invalid or undefined dates
                if ($task->taskend == -1){
                    if ($task->milestoneid){
                        $milestone = get_record('techproject_milestone', 'id', $task->milestoneid);
                        if ($milestone->deadline > 0) $task->taskend = $milestone->deadline;
                    }
                }
                if ($task->taskend == -1) $task->taskend = $project->projectend;
    
                print_date_selector('taskendday', 'taskendmonth', 'taskendyear', $task->taskend);
                echo "&nbsp;-&nbsp;";
                print_time_selector('taskendhour', 'taskendminute', $task->taskend);
                helpbutton('taskenddate', get_string('taskenddate', 'techproject'), 'techproject', true, false);
        ?>
            <input type="hidden" name="htaskendday"    value="0" />
            <input type="hidden" name="htaskendmonth"  value="0" />
            <input type="hidden" name="htaskendyear"   value="0" />
            <input type="hidden" name="htaskendhour"   value="0" />
            <input type="hidden" name="htaskendminute" value="0" />
        </td>
    </tr>
    
    <tr valign="top">
    	<td align="right"><b><?php print_string('worktype', 'techproject') ?>:</b>
    	</td>
        <td align="left">
    <?php
            $worktypes = techproject_get_options('techproject_worktype', $project->id);
            $worktypeoptions = array();
            foreach($worktypes as $aWorktype){
                $worktypeoptions[$aWorktype->worktype] = '['. $aWorktype->worktype . '] ' . $aWorktype->label;
            }
            choose_from_menu ($worktypeoptions, 'worktype', $task->worktype);
            helpbutton('worktype', get_string('worktype', 'techproject'), 'techproject', true, false);
    ?>
        </td>
    </tr>
    <tr valign="top">
    	<td align="right">
    	   <b><?php print_string('status', 'techproject') ?>:</b>
    	</td>
        <td align="left">
    <?php
        $statusses = techproject_get_options('techproject_taskstatus', $project->id);
        $statussesoptions = array();
        foreach($statusses as $aStatus){
            $statussesoptions[$aStatus->taskstatus] = '['. $aStatus->taskstatus . '] ' . $aStatus->label;
        }
        choose_from_menu ($statussesoptions, 'status', $task->status);
        helpbutton('status', get_string('status', 'techproject'), 'techproject', true, false);
    ?>
        </td>
    </tr>
    
    <tr valign="top">
    	<td align="right"><b><?php print_string('costrate', 'techproject') ?>:</b></td>
        <td align="left">
            <input type="text" name="costrate" size="3" value="<?php p($task->costrate) ?>" alt="<?php  print_string('costrate', 'techproject') ?>" />
        </td>
    </tr>
    <tr valign="top">
    	<td align="right"><b><?php print_string('costplanned', 'techproject') ?>:</b></td>
        <td align="left">
    <?php
        // we can only edit planned indicator if we are a leaf task.
        if (techproject_count_subs('techproject_task', $taskid) == 0){
            echo "<input type=\"text\" name=\"planned\" size=\"3\" value=\"{$task->planned}\" alt=\"".get_string('costplanned', 'techproject')."\" /> ".$TIMEUNITS[$project->timeunit];
        }
        else {
            echo "{$task->planned}<input type=\"hidden\" name=\"planned\" size=\"3\" value=\"{$task->planned}\" /> ".$TIMEUNITS[$project->timeunit]; 
        }
    ?>
            <?php helpbutton('costplanned', get_string('cost', 'techproject'), 'techproject', true, false); ?>
        </td>
    </tr>
    <tr valign="top">
    	<td align="right"><b><?php print_string('done', 'techproject') ?>:</b></td>
        <td align="left">
    <?php
        // we can only edit done indicator if we are a leaf task.
        if (techproject_count_subs('techproject_task', $taskid) == 0){
            echo "<input type=\"text\" name=\"done\" size=\"3\" value=\"{$task->done}\" alt=\"".get_string('done', 'techproject')."\" />"; 
        }
        else {
            echo "{$task->done}<input type=\"hidden\" name=\"done\" size=\"3\" value=\"{$task->done}\" />"; 
        }
    ?>
            %
            <?php helpbutton('done', get_string('done', 'techproject'), 'techproject', true, false); ?>
        </td>
    </tr>
    <tr valign="top">
    	<td align="right"><b><?php print_string('costelapsed', 'techproject') ?>:</b></td>
        <td align="left">
    <?php
        // we can only edit planned indicator if we are a leaf task.
        if (techproject_count_subs('techproject_task', $taskid) == 0){
            echo "<input type=\"text\" name=\"cost\" size=\"3\" value=\"{$task->cost}\" alt=\"".get_string('costelapsed', 'techproject')."\" /> ".$TIMEUNITS[$project->timeunit];
        }
        else {
            echo "{$task->cost}<input type=\"hidden\" name=\"cost\" size=\"3\" value=\"{$task->cost}\" /> ".$TIMEUNITS[$project->timeunit]; 
        }
    ?>
            <?php helpbutton('costelapsed', get_string('costelapsed', 'techproject'), 'techproject', true, false); ?>
        </td>
    </tr>
    <tr valign="top">
        <td align="right"><b><?php print_string('description', 'techproject') ?>:</b><br />
        <font size="1">
         <?php
            helpbutton('writing', get_string('helpwriting'), 'moodle', true, true);
            echo "<br />";
            if ($usehtmleditor) {
               helpbutton('richtext', get_string('helprichtext'), 'moodle', true, true);
            } else {
               helpbutton('text', get_string('helptext'), 'moodle', true, true);
               echo "<br />";
               emoticonhelpbutton('updatetaskform', 'description', 'moodle', true, true);
               echo "<br />";
            }
          ?>
          <br />
        </font>
        </td>
        <td>
        <?php
           print_textarea($usehtmleditor, 20, 60, 595, 400, 'description', $task->description);
    
           if ($usehtmleditor) {
               echo '<input type="hidden" name="format" value="'.FORMAT_HTML.'" />';
               $nohtmleditorneeded = false;
           } else {
               echo '<p align="right">';
               helpbutton('textformat', get_string('formattexttype'));
               print_string('formattexttype');
               echo ':&nbsp;';
               if (empty($task->format)) {
                   $task->format = $defaultformat;
               }
               choose_from_menu(format_text_menu(), 'format', $task->format, '');
               echo '</p>';
           }
        ?>
        </td>
    </tr>
    <tr valign="top">
    	<td align="right"><b><?php print_string('milestone', 'techproject') ?>:</b>
    	</td>
        <td align="left">
    <?php
        $milestones = get_records_select('techproject_milestone', "projectId = {$project->id} AND groupId = {$currentGroupId}", 'ordering ASC', 'id, abstract, ordering');
        $milestonesoptions = array();
        if ($milestones){
            foreach($milestones as $aMilestone){
                $milestonesoptions[$aMilestone->id] = format_string($aMilestone->abstract);
            }
            choose_from_menu ($milestonesoptions, 'milestoneid', $task->milestoneid, get_string('unassigned', 'techproject') );
        }
        else{
            print_string('nomilestones', 'techproject');
        }
        helpbutton('tasktomilestone', get_string('milestone', 'techproject'), 'techproject', true, false);
    ?>
        </td>
    </tr>
    <tr valign="top">
    	<td align="right"><b><?php print_string('tasktospec', 'techproject') ?>:</b>
    	</td>
        <td align="left">
            <select name="tasktospec[]" multiple="multiple" size="6" style="width:100%">
    <?php
            $specifications = techproject_get_tree_options('techproject_specification', $project->id, $currentGroupId);
            $selection = get_records_select_menu('techproject_task_to_spec', "taskId = $taskid", '', 'specId,taskId');
            foreach($specifications as $aSpec){
                $aSpec->abstract = format_string($aSpec->abstract);
                $selected = ($selection && in_array($aSpec->id, array_keys($selection))) ? "selected=\"selected\"" : '' ;
                echo "<option value=\"{$aSpec->id}\" $selected>{$aSpec->ordering} - {$aSpec->abstract}</option>";
            }
    ?>
            </select>
            <?php helpbutton('tasktospecification', get_string('tasktospec', 'techproject'), 'techproject', true, false); ?>
        </td>
    </tr>
    <tr valign="top">
    	<td align="right"><b><?php print_string('taskdependency', 'techproject') ?>:</b>
    	</td>
        <td align="left">
            <select name="taskdependency[]" multiple="multiple" size="6" style="width:100%">
    <?php
            $tasks = techproject_get_tree_options('techproject_task', $project->id, $currentGroupId);
            $selection = get_records_select_menu('techproject_task_dependency', "slave = $taskid", '', 'master,slave');
            foreach($tasks as $aTask){
                $aTask->abstract = format_string($aTask->abstract);
                if ($aTask->id == $taskid) continue;
                if ($aTask->id == $task->fatherid) continue;
                if (techproject_check_task_circularity($taskid, $taskid)) continue;
                $selected = ($selection && in_array($aTask->id, array_keys($selection))) ? "selected=\"selected\"" : '' ;
                echo "<option value=\"{$aTask->id}\" {$selected}>{$aTask->ordering} - {$aTask->abstract}</option>";
            }
    ?>
            </select>
            <?php helpbutton('taskdependency', get_string('taskdependency', 'techproject'), 'techproject', true, false); ?>
        </td>
    </tr>
    <tr valign="top">
    	<td align="right"><b><?php print_string('tasktodeliv', 'techproject') ?>:</b>
    	</td>
        <td align="left">
            <select name="tasktodeliv[]" multiple="multiple" size="6" style="width:100%">
    <?php
            $deliverables = techproject_get_tree_options('techproject_deliverable', $project->id, $currentGroupId);
            $selection = get_records_select_menu('techproject_task_to_deliv', "taskId = $taskid", '', 'delivId,taskId');
            foreach($deliverables as $aDeliv){
                $aDeliv->abstract = format_string($aDeliv->abstract);
                $selected = ($selection && in_array($aDeliv->id, array_keys($selection))) ? "selected=\"selected\"" : '' ;
                echo "<option value=\"{$aDeliv->id}\" $selected>{$aDeliv->ordering} - {$aDeliv->abstract}</option>";
            }
    ?>
            </select>
            <?php helpbutton('tasktodeliv', get_string('tasktodeliv', 'techproject'), 'techproject', true, false); ?>
        </td>
    </tr>
    </table>
    <input type="button" name="go_btn" value="<?php print_string('savechanges'); ?>" onclick="senddata();" />
    <input type="button" name="cancel_btn" value="<?php print_string('cancel'); ?>" onclick="cancel();" />
    </form>
    <script type="text/javascript">
    //<![CDATA[
    <?php
        if (!$task->taskstartenable) echo "lockoptions('updatetaskform','taskstartenable', taskstartitems);\n";
        if (!$task->taskendenable) echo "lockoptions('updatetaskform','taskendenable', taskenditems);\n";
    ?>
    //]]>
    </script>
    </center>
    
    <?php
    		}
    	}

/// Delete form *********************************************************

    	elseif ($work == 'delete') {
    	    $taskid = required_param('taskid', PARAM_INT);
    ?>
    
    <center>
    <?php print_heading(get_string('deletetask','techproject'), 'center'); ?>
    <script type="text/javascript">
    //<![CDATA[
    function senddata(){
        document.forms['deletetaskform'].work.value='dodelete';
        document.forms['deletetaskform'].submit();
    }
    
    function cancel(){
        document.forms['deletetaskform'].submit();
    }
    //]]>
    </script>
    <form name="deletetaskform" method="get" action="view.php">
    <input type="hidden" name="id" value="<?php p($cm->id) ?>" />
    <input type="hidden" name="work" value="" />
    <input type="hidden" name="taskid" value="<?php p($taskid) ?>" />
    <input type="button" name="go_btn" value="<?php print_string('delete') ?>" onclick="senddata();" />
    <input type="button" name="cancel_btn" value="<?php print_string('cancel') ?>" onclick="cancel();" />
    </form>
    </center>
    
    <?php
    	}

/// Group operation form *********************************************************

    	elseif ($work == "groupcmd") {
    	    $ids = required_param('ids', PARAM_INT);
    	    $cmd = required_param('cmd', PARAM_ALPHA);
    ?>
    
    <center>
    <?php 
    print_heading(get_string('groupoperations', 'techproject'), 'center');
    print_heading(get_string("group$cmd", 'techproject'), 'center', 'h3');
    if ($cmd == 'copy' || $cmd == 'move')
        print_simple_box(get_string('groupcopymovewarning', 'techproject'), 'center', '70%'); 
    ?>
    <script type="text/javascript">
    //<![CDATA[
    function senddata(cmd){
        document.forms['groupopform'].work.value="do" + cmd;
        document.forms['groupopform'].submit();
    }
    
    function cancel(){
        document.forms['groupopform'].submit();
    }
    //]]>
    </script>
    <form name="groupopform" method="get" action="view.php">
    <input type="hidden" name="id" value="<?php p($cm->id) ?>" />
    <input type="hidden" name="work" value="" />
    <?php
            foreach($ids as $anId){
                echo "<input type=\"hidden\" name=\"ids[]\" value=\"{$anId}\" />\n";
            }
            if (($cmd == 'move')||($cmd == 'copy')){
                echo get_string('to', 'techproject');
                $options['requs'] = get_string('requirements', 'techproject');
                $options['specs'] = get_string('specifications', 'techproject');
                $options['deliv'] = get_string('deliverables', 'techproject');
                choose_from_menu($options, 'to', '', 'choose');
            }
    ?>
    <input type="button" name="go_btn" value="<?php print_string('continue') ?>" onclick="senddata('<?php p($cmd) ?>')" />
    <input type="button" name="cancel_btn" value="<?php print_string('cancel') ?>" onclick="cancel()" />
    </form>
    </center>
    
    <?php
    	}
    	else {
    ?>
    <script type="text/javascript">
    //<![CDATA[
    function sendgroupdata(){
        document.forms['groupopform'].submit();
    }
    //]]>
    </script>
    <form name="groupopform" method="post" action="view.php">
    <input type="hidden" name="id" value="<?php p($cm->id) ?>" />
    <input type="hidden" name="work" value="groupcmd" />
    <?php
            if ($USER->editmode == 'on' && has_capability('mod/techproject:changetasks', $context)) {
        		echo "<br/><a href='view.php?id={$cm->id}&amp;work=add&amp;fatherid=0'>".get_string('addroottask','techproject')."</a> ";
    	    }
    		techproject_print_tasks($project, $currentGroupId, 0, $cm->id);
            if ($USER->editmode == 'on' && has_capability('mod/techproject:changetasks', $context)) {
        		echo "<br/><a href='view.php?id={$cm->id}&amp;work=add&amp;fatherid=0'>".get_string('addroottask','techproject')."</a> ";
        		techproject_print_group_commands('markasdone,fullfill');
    	    }
    ?>
    </form>
<?php
	}
?>